@@ -7,7 +7,8 @@
 
 Compass::Compass(const Cache &cache, QWidget *parent) : 
   QWidget(parent), m_hasCompass(false), m_timer(NULL), 
-  m_bgPix(NULL), m_calPix(NULL), m_cache(cache) {
+  m_bgPix(NULL), m_calPix(NULL), m_cache(cache),
+  m_target(cache.coordinate()) {
   
   qDebug() << __FUNCTION__;
 
@@ -60,8 +61,18 @@ void Compass::setAzimuth(qreal azimuth) {
   }
 }
 
+void Compass::setTarget(const QGeoCoordinate &pos) {
+  if(pos.isValid() && m_lastPos.isValid()) {
+    m_target = pos;
+
+    m_needleAngle = m_lastPos.azimuthTo(m_target);
+    update();
+  }
+}
+
 void Compass::setPosition(const QGeoPositionInfo &pos) {
   if(pos.isValid() && pos.coordinate().isValid()) {
+    m_lastPos = pos.coordinate();
     
     if(!m_hasCompass) {
       if(pos.hasAttribute(QGeoPositionInfo::Direction) ||
@@ -75,7 +86,7 @@ void Compass::setPosition(const QGeoPositionInfo &pos) {
       }
     }
     
-    m_needleAngle = pos.coordinate().azimuthTo(m_cache.coordinate());
+    m_needleAngle = pos.coordinate().azimuthTo(m_target);
     update();
   }
 }
