@@ -7,6 +7,7 @@
 #include <math.h>
 
 #include "mapwidget.h"
+#include "custompixmap.h"
 #include "config.h"
 
 #if(QTM_VERSION < 0x010100)
@@ -23,7 +24,7 @@ void MapWidget::updateZoomButtons() {
 
 MapWidget::MapWidget(QGeoMappingManager *mgr) : 
   QGraphicsGeoMap(mgr), m_bubble(NULL), m_posIsValid(false), m_gpsMarker(NULL),
-  m_manager(mgr) {
+  m_manager(mgr), m_wptDb(NULL) {
   
   // try to get default from qsettings, otherwise start at berlin's zoo
   QSettings settings;
@@ -35,10 +36,12 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
   setMapType((QGraphicsGeoMap::MapType)settings.value("Type", 
 			      QGraphicsGeoMap::StreetMap).toInt());
   m_gpsLocked = settings.value("GpsLocked", false).toBool();
+  m_hiRez = settings.value("HiRez", false).toBool();
   m_precisionAlpha = settings.value("PrecisionAlpha", 32).toInt();
   settings.endGroup();
-  m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE);
-  
+  if(m_hiRez) m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE_HIREZ);
+  else        m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE);
+
   // create the control overlays
   this->m_mapOverlay = new MapOverlay(this);
   this->m_mapBanner = new MapBanner(this);
@@ -55,6 +58,7 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
   addMapObject(m_cacheGroup);
   
   // create a gps marker and precision indicator, but hide them
+  m_compass = NAN;
   QPixmap *markerIcon = m_iconLoader->load("gps_marker");
   if(markerIcon) {
     m_gpsMarker = new QGeoMapPixmapObject(QGeoCoordinate(0,0), 
@@ -251,7 +255,11 @@ void MapWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
 
     if(!clickedCache.isEmpty()) {
       m_selectedCache = clickedCache;
-      
+
+      // update list of custom waypoints for the current cache
+      if(m_wptDb)
+	m_customWpts =  m_wptDb->get(clickedCache);
+     
       /* update caches to show waypoints of selected cache if present */
       emit mapChanged();
  
@@ -289,15 +297,12 @@ void MapWidget::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 }
 
 MapWidget::MapCacheObject::MapCacheObject(IconLoader *loader, const Cache &cache, bool highlight) :
-  QGeoMapPixmapObject(cache.coordinate(), 
-		      QPoint(-loader->size()/2, -loader->size()/2),
-		      *loader->load(cache)) {
+  QGeoMapPixmapObject(cache.coordinate()) {
 
-  if(highlight) {
-    QPixmap *hlPix = loader->loadHighlight(cache);
-    setOffset(QPoint(-hlPix->width()/2, -hlPix->height()/2));
-    setPixmap(*hlPix);
-  }
+  CustomPixmap *pix = 
+    highlight?loader->loadHighlight(cache):loader->load(cache);
+  setOffset(pix->offset());
+  setPixmap(*pix);
 
   setZValue(0);  // below gps marker and bubble
   setObjectName("cache");
@@ -313,9 +318,14 @@ bool MapWidget::MapCacheObject::contains(const QGeoCoordinate &coo) const {
 
 MapWidget::MapWaypointObject::MapWaypointObject(IconLoader *loader, 
 		const QString &cacheName, const Waypoint &waypoint) :
-  QGeoMapPixmapObject(waypoint.coordinate(),
-		      QPoint(-loader->size()/3, -loader->size()/3),
-		      *loader->load(waypoint.iconFile(), 2*loader->size()/3)) {
+  QGeoMapPixmapObject(waypoint.coordinate()) {
+
+  // set icon
+  CustomPixmap *pix = 
+    loader->load(waypoint.iconFile(), 2*loader->size()/3); 
+
+  setPixmap(*pix);
+  setOffset(pix->offset());
   
   setZValue(1);  // below gps marker and bubble, but above caches
   setObjectName("waypoint");
@@ -355,7 +365,7 @@ void MapWidget::updateCaches(const CacheList &cacheList) {
 
     // highlight if cache has waypoints
     m_cacheGroup->addChildObject(new MapCacheObject(m_iconLoader, *i, 
-		    isSelected && i->waypoints().size() > 0));
+     isSelected && ((i->waypoints().size() > 0) || (m_customWpts.size() > 0))));
 				 
     // also draw waypoints of selected cache
     if(isSelected) 
@@ -366,6 +376,9 @@ void MapWidget::updateCaches(const CacheList &cacheList) {
   foreach(Waypoint wpt, m_selectedWaypoints)
     m_cacheGroup->addChildObject(new MapWaypointObject(m_iconLoader, m_selectedCache, wpt));
 
+  // draw all custom waypoints
+  foreach(Waypoint wpt, m_customWpts)
+    m_cacheGroup->addChildObject(new MapWaypointObject(m_iconLoader, m_selectedCache, wpt));
 
   // restore previous bubble
   if(bubbleCache.coordinate().isValid()) {
@@ -413,6 +426,28 @@ void MapWidget::reload() {
     this->m_timer->setInterval(900);
 }
 
+void MapWidget::positionUpdated(const QCompassReading *reading) {
+  // we accept 90% calibration
+  if(reading->calibrationLevel() > 0.9) {
+    qreal newVal = reading->azimuth();
+    if(newVal != m_compass) {
+      QPixmap *markerIcon = m_iconLoader->load("gps_arrow");
+
+      // todo: rotate markerIcon
+
+      if(m_gpsMarker) m_gpsMarker->setPixmap(*markerIcon);
+      m_compass = newVal;
+    }
+  } else {
+    if(!isnan(m_compass)) {
+      QPixmap *markerIcon = m_iconLoader->load("gps_marker");
+      if(m_gpsMarker) m_gpsMarker->setPixmap(*markerIcon);
+
+      m_compass = NAN;
+    }
+  }
+}
+
 void MapWidget::positionUpdated(const QGeoPositionInfo &pos) {
   // change state of gps button to reflect gps availability
   if(m_posIsValid != pos.coordinate().isValid()) {
@@ -468,3 +503,44 @@ void MapWidget::setBusy(bool on) {
     m_mapSpinner = NULL;
   }
 }
+
+bool MapWidget::hiRez() const {
+  return m_hiRez;
+}
+
+void MapWidget::setHiRez(bool hiRez) {
+  if(m_hiRez != hiRez) {
+    m_hiRez = hiRez;
+
+    delete m_iconLoader;
+    if(m_hiRez) 
+      m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE_HIREZ);
+    else
+      m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE);
+
+    // this will also cause the overlay to redraw
+    emit mapChanged();
+  }
+}
+
+void MapWidget::reloadCustomWaypoints() {
+  // waypointsDb has signalled that the waypoints have changed
+  bool hadCustomWpts = (m_customWpts.size() != 0);
+
+  if(m_wptDb && !m_selectedCache.isEmpty()) {
+    m_customWpts =  m_wptDb->get(m_selectedCache);
+
+    // if there weren't any custom waypoints and there now aren't
+    // we have nothing to redraw
+    if((m_customWpts.size() != 0) || hadCustomWpts) 
+      emit mapChanged();  
+  }
+}
+
+void MapWidget::setWaypointDb(WaypointDb *wptDb) {
+  qDebug() << __FUNCTION__;
+  m_wptDb = wptDb;
+
+  QObject::connect(this->m_wptDb, SIGNAL(waypointsChanged()),
+		   this, SLOT(reloadCustomWaypoints()));
+}
